'''
baseShellAttackPlugin.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

from core.controllers.w3afException import w3afException
from core.controllers.basePlugin.baseAttackPlugin import baseAttackPlugin
import core.controllers.outputManager as om
from core.controllers.vdaemon.vdFactory import *
from core.controllers.w3afAgent.w3afAgentManager import w3afAgentManager
import time
import core.data.parsers.urlParser

class baseShellAttackPlugin(baseAttackPlugin):
	'''
	This is the base class for SHELL attack plugins, all SHELL attack plugins should inherit from it 
	and implement the following methods :
		1. _rexec(...)
		
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseAttackPlugin.__init__( self )

	def rexec( self, command ):
		'''
		This is the method that is called when a user want's to execute something in the remote operating system.
		What I do here is first trap the requests for starting the virtual daemon and the w3afAgent, and this ain't the
		case I forward the request to the _rexec method, that should be implemented by all shellAttackPlugins.
		'''
		if command == 'start vdaemon':
			# start a vdaemon!
			#om.out.console('')
			vd = getVirtualDaemon(self._rexec)
			vd.setRemoteIP( urlParser.getDomain( self._vuln.getURL() )  )
			vd.start2()
			# Let the server start
			time.sleep(0.1)
			return ''
		elif command == 'start w3afAgent':
			# start a w3afAgent, to do this, I must transfer the agent client to the
			# remote end and start the w3afServer in this local machine
			# all this work is done by the w3afAgentManager, I just need to called
			# start and thats it.
			agentManager = w3afAgentManager(self._rexec)
			agentManager.run()
			return ''
		elif hasattr( self, '_rexec'):
			# forward to the plugin
			return self._rexec( command )
		else:
			raise w3afException('Plugins inhereting from baseShellAttackPlugin should implement the _rexec method.')
