'''
w3afTest.py.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

import os
import commands
import core.controllers.outputManager as om

def getScripts():
	res = []
	withOutAssert = []
	for f in os.listdir('scripts/'):
		if f.endswith('.w3af'):
			content = file( 'scripts/' + f ).read()
			if 'assert' in content:
				res.append( f )
			else:
				withOutAssert.append( f )
	return res, withOutAssert

def runScript( scriptName ):
	om.out.information('Running: ' + scriptName )
	try:
		output = commands.getoutput('python w3af -s scripts/' + scriptName)
	except KeyboardInterrupt, k:
		output = ''
		om.out.information('User cancelled the test.')
	return output
	
def analyzeResult( resultString ):
	lines = resultString.split('\n')
	for line in lines:
		if 'Assert **FAILED**' in line:
			om.out.error( line )
		elif 'Traceback (most recent call last):' in line:
			om.out.error( 'An unhandled exception was raised during the execution of this script!' )
	
def w3afTest():
	'''
	Test all scripts that have an assert call.
	'''
	assertScriptList, scriptsWithoutAssert = getScripts()
	for assertScript in assertScriptList:
		try:
			result = runScript( assertScript )
			analyzeResult( result )
		except:
			pass
	
	om.out.console( '')
	om.out.console( 'Results:')
	om.out.console( '========')
	om.out.console( '- ' + str(len(assertScriptList)) + ' scripts have been tested.')
	om.out.console( '- ' + str(len(scriptsWithoutAssert)) + ' scripts don\'t have assert statements. This is the list of scripts without assert statements:')
	om.out.console( '- ' + ' , '.join(scriptsWithoutAssert) )
