'''
findvhost.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

import core.controllers.outputManager as om
from core.controllers.basePlugin.baseDiscoveryPlugin import baseDiscoveryPlugin
import core.data.kb.knowledgeBase as kb
import core.data.kb.vuln as vuln
import core.data.parsers.urlParser as urlParser
from core.controllers.w3afException import *

class findvhost(baseDiscoveryPlugin):
	'''
	This plugin modifies the HTTP Host header and tries to find virtual hosts.
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseDiscoveryPlugin.__init__(self)
		self._exec = True
		
	def discover(self, fuzzableRequest ):
		'''
		Find virtual hosts.
		
		@parameter fuzzableRequest: A fuzzableRequest instance that contains (among other things) the URL to test.
		'''
		if not self._exec :
			# This will remove the plugin from the discovery plugins to be runned.
			raise w3afRunOnce()
			
		else:
			# Only run once
			self._exec = False
			
			baseURL = urlParser.baseUrl( fuzzableRequest.getURL() )
			
			commonVhostList = self._getCommonVirtualHosts( urlParser.getDomain( baseURL ) )
			
			# Get some responses to compare later
			originalResponse = self._urlOpener.GET( baseURL, useCache=True )
			nonExistantResponse = self._urlOpener.GET( baseURL, useCache=True, headers={'Host': 'iDoNotExistPleaseGoAwayNowOrDie' } )
			
			for commonVhost in commonVhostList:
				try:
					vhostResponse = self._urlOpener.GET( baseURL, useCache=True, headers={'Host': commonVhost } )
				except w3afException, w3:
					pass
				else:
					if vhostResponse.getBody() != originalResponse.getBody() and \
					vhostResponse.getBody() != nonExistantResponse.getBody():
						v = vuln.vuln()
						v.setURL( originalResponse.getURL() )
						v.setMethod( 'GET' )
						v.setDesc('Found a new virtual host at the target web server, virtual host name is: "' + commonVhost + '"' )
						v.setId( vhostResponse.id )
						kb.kb.append( self, 'findvhost', v )
						om.out.information( v.getDesc() )		
				
		return []
	
	def _getCommonVirtualHosts( self, domain ):
		res = []
		
		for subdomain in ['intranet', 'intra', 'extranet', 'extra' , 'test' , 'old' , 'new' , 'admin' ]:
			# intranet
			res.append( subdomain )
			# intranet.www.targetsite.com
			res.append( subdomain + domain )
			# intranet.targetsite.com
			res.append( subdomain + urlParser.getRootDomain( domain ) )
			# This is for:
			# intranet.targetsite
			res.append( subdomain + urlParser.getRootDomain( domain ).split('.')[0] )
		
		return res

	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		pass

	def getPluginDeps( self ):
		'''
		@return: A list with the names of the plugins that should be runned before the
		current one.
		'''
		return []
		
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This plugin uses the HTTP Host header to find new virtual hosts. For example, if the intranet page is hosted
		in the same server that the public page, and the web server is misconfigured, this plugin will discover that
		virtual host.
		
		Please note that this plugin doesn't use any DNS technique to find this virtual hosts.
		'''
