'''
fullWidthEncode.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

from core.controllers.basePlugin.baseEvasionPlugin import baseEvasionPlugin
from core.controllers.w3afException import w3afException
from random import choice, randint
import urllib

class fullWidthEncode(baseEvasionPlugin):
	'''
	This evasion plugin does full width encoding.
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseEvasionPlugin.__init__(self)

	def fuzzUrl(self, path ):
		'''
		Mangles the URI.
		
		@parameter url: A string containing the Url to mangle ( '/bar/foo.php' )
		'''
		# This is a test URL
		# http://172.16.1.132/index.asp?q=%uFF1Cscript%3Ealert(%22Hello%22)%3C/script%3E
		# This is the content of index.asp :
		# <%=Request.QueryString("q")%>

		path = urllib.unquote( path )
		newpath = ''
		for char in path:
			if char not in ['?', '/', '&', '\\', '=', '%', '+']:
				# The "- 0x20" was taken from UFF00.pdf
				char = "%%uFF%02x" % ( ord(char) - 0x20 )
			newpath += char
		
		return newpath
	
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/ui/userInterface.dtd
		
		@return: XML with the plugin options.
		'''	
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def setOptions( self, OptionList ):
		'''
		This method sets all the options that are configured using the user interface 
		generated by the framework using the result of getOptionsXML().
		
		@parameter OptionList: A dictionary with the options for the plugin.
		@return: No value is returned.
		'''	
		pass
		
	def getPluginDeps( self ):
		return []

	def getPriority( self ):
		'''
		This function is called when sorting evasion plugins.
		Each evasion plugin should implement this.
		
		@return: An integer specifying the priority. 0 is runned first, 100 last.
		'''
		return 50
	
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This evasion plugin does full width encoding as described here:
			- http://www.kb.cert.org/vuls/id/739224
		
		Example:
			Input:		'/bar/foo.asp'
			Output :	'/b%uFF61r/%uFF66oo.asp'
		'''
